A simple and easy AJAX framework for Java web development.




Download the latest version
===============================================================================

https://github.com/corgrath/AjaxPostBackServlet/downloads/



Quick technical description
===============================================================================

AjaxPostBackServlet is a lightweight AJAX framework for Java web development.

The core logic of the framework is that the Java developer defines a class that extends the *AjaxPostBackEvent* class. 
In HTML he makes a call to the event with any given of arguments, which gets sent to the browser.

When a web user triggers the call-JavaScript on the page, an AJAX request gets sent back to the browser with what event should be triggered and with what arguments.
The server then invokes the class and provides the arguments as constructor parameters. The object returns JavaScript, which gets sent back to the browser to be executed.

![Flowchart](http://dl.dropbox.com/u/8183146/persistent/projects/java_ajaxpostbackservlet/flowchart.png "Flowchart")



Quick guide - 8 simple steps
===============================================================================

Here is a quick guide for common usage.

1 Download and include the JavaScript
-------------------------------------------------------------------------------

Download the latest version of the JavaScript source code and attach it to the project. 

2 Download and include the JAR-library
-------------------------------------------------------------------------------

Download the latest version of the Java library (the JAR-file) and attach it to the project's class path.

3 Extend a servlet with AjaxPostBackServlet
-------------------------------------------------------------------------------

Create a servlet class that extends AjaxPostBackServlet. An example would be:

	public class ServletAjaxPostBack extends AjaxPostBackServlet {
	
		private static final long serialVersionUID = 1L;
	
		@Override
		protected Map<Serializable, Object> getMetaData(final HttpServletRequest request, final HttpServletResponse response) {
		}
	
		@Override
		protected JavaScript getExceptionJavaScriptResponse(String eventName, String[] arguments, long duration, Exception exception) {
	
			String errorMessage = exception.toString();
			errorMessage = StringEscapeUtils.escapeHtml4(errorMessage);
			errorMessage = errorMessage.replaceAll("\r|\n", "");
	
			return new JavaScript("alert(\"" + errorMessage + " when calling '" + eventName + "'. Request took '" + duration + "' ms.\");");
	
		}
	
		@Override
		protected void onSuccess(String eventName, String[] arguments, long duration, JavaScript response) {
			System.out.println("Success for " + eventName + ". Request took '" + duration + "' ms.");
		}
	
		@Override
		protected void onError(String eventName, String[] arguments, long duration, JavaScript response) {
			System.out.println("Error for " + eventName + ". Request took '" + duration + "' ms.");
		}
	
	}
	
4 Modify the JavaScript file
-------------------------------------------------------------------------------

Open the recently downloaded JavaScript file and change the *$apb.url* so it points to the servlet.

	// The URL path to the AjaxPostBackServlet
	$apb.url = "/path_to_my_newly_created_servlet/";

5 Create a caller class
-------------------------------------------------------------------------------

In order to call events, you have to create your own caller, which is done by creating a class that extends AjaxPostBackCaller. An example would be:

	public final class SimpleCaller extends AjaxPostBackCaller {
	
		private static final long serialVersionUID = 1L;
	
		public SimpleCaller(final Class<? extends AjaxPostBackEvent> eventClass, final Serializable... arguments) {
			super(null, null, eventClass, arguments);
		}
	
	}

6 Create your first event
-------------------------------------------------------------------------------

Create a class that extends AjaxPostBackEvent.

	@IsAjaxPostBackEvent
	public final class HelloEvent extends AjaxPostBackEvent {
	
		private String name;
	
		public HelloEvent(final String name) {
			this.name = name;
		}
	
		@Override
		public JavaScript getJavaScriptResponse() {
			return new JavaScript("alert('Hello " + name + "!');");	
		}
	
		public static AjaxPostBackCaller call(final String name) {
			return new SimpleCaller(HelloEvent.class, name);
		}
	
	}


7 Adding the call in HTML
-------------------------------------------------------------------------------

	out.println( "<button type=\"button\" onclick=\"" + MyEvent.call("John") + "\">Click Me!</button>" );

Which outputs something like:

	<button type="button" onclick="$apb('','','src.HelloEvent',['John']);">Click Me!</button>


8 Click the button
-------------------------------------------------------------------------------

When a visitor clicks on the button, an AJAX request will be sent to the servlet and an alert box will appear.


Frequently asked Questions
===============================================================================

How do include internal specific data to each event?
-------------------------------------------------------------------------------

You add meta data with the AjaxPostBackServlet.getMetaData method. An example would be:

	public class ServletAjaxPostBack extends AjaxPostBackServlet {
	
		private static final long serialVersionUID = 1L;
	
		@Override
		protected Map<Serializable, Object> getMetaData(final HttpServletRequest request, final HttpServletResponse response) {
		
			Map<Serializable, Object> metaData = new HashMap<Serializable, Object>();
	
			metaData.put("request", request);
			metaData.put("session", new SessionInformation(request));
	
			return metaData;
		
		}
		
		...
		
	}
		
And the get the data inside an event:

	@IsAjaxPostBackEvent
	public class LogoutEvent extends AjaxPostBackEvent {
	
		public JavaScript getJavaScriptResponse() {
	
			// Get the session information
			SessionInformation session = (SessionInformation) getMetaData(ServletAjaxPostBack.MetaDataKeys.SESSION);
			
			...
			
		}
		
		...
		
	}


Is it possible to show a "Loading..." text, during the AJAX call?
-------------------------------------------------------------------------------

Yes, but it requires that you have implemented a "Loading..." logic yourself in JavaScript.

Lets assume you have written two JavaScript functions:

 * showLoadingText(); - The JavaScript function that shows the "Loading..." text 
 * hideLoadingText(); - The JavaScript function that hides the "Loading..." text
 
Then you can create a more advanced AjaxPostBackCaller. Here is an example:

	public class LoadingCaller extends AjaxPostBackCaller {
	
		private static final long serialVersionUID = 1L;
	
		public LoadingCaller(final Class<? extends AjaxPostBackEvent> eventClass, final Serializable... arguments) {
			super(new JavaScript("showLoadingText();"), new JavaScript("hideLoadingText();"), eventClass, arguments);
		}
	
	}
  
And then use it in your event like this:

	public static AjaxPostBackCaller call(final String name) {
		return new LoadingCaller(HelloEvent.class, name);
	}

Is it possible to send information fetched with JavaScript to a AjaxPostBackEvent?
-------------------------------------------------------------------------------

Yes, but you need to set the data type in the call-method to JavaScript. Here is an example:

Given there is an event like this:

	@IsAjaxPostBackEvent
	public final class HelloEvent extends AjaxPostBackEvent {
	
		private String name;
	
		public HelloEvent(final String name) {
			this.name = name;
		}
	
		@Override
		public JavaScript getJavaScriptResponse() {
			return new JavaScript("alert('Hello " + name + "!');");	
		}
	
		public static AjaxPostBackCaller call(final JavaScript name) {
			return new SimpleCaller(HelloEvent.class, name);
		}
	
	}

Note that even though the constructor has a String parameter:

	public HelloEvent(final String name) {
		
The call method has a JavaScript parameter:

	public static AjaxPostBackCaller call(final JavaScript name) {
	
It is possible to fetch information from, as an example, an HTML input, like this:

	<input type="text" name="fname" value="John" />
	
With this code:

	JavaScript getNameFromInputScript = new JavaScript("document.getElementById('fname').value");

	out.println( "<button type=\"button\" onclick=\"" + HelloEvent(getNameFromInputScript) + "\">Click Me!</button>" );
	
Which translates into:

	<button type="button" onclick="$apb('','','src.HelloEvent',[document.getElementById('fname').value]);">Click Me!</button>	
 

What data types does the framework support for the events?
-------------------------------------------------------------------------------

All primitive data types and the JavaScript type. For example:

	@IsAjaxPostBackEvent
	public class TestEvent extends AjaxPostBackEvent {
	
		public TestEvent( String s, long l, int i, double d, float f, JavaScript js ) {
		
		... 
		
		}
		
	...
	
	}


Is there a more advanced example you can show?
-------------------------------------------------------------------------------
Here is an example of an event that shows a JQuery dialog with input fields in it:

	@IsAjaxPostBackEvent
	final public class ShowLoginDialogEvent extends AjaxPostBackEvent {
	
		public static AjaxPostBackCaller call() {
			return new SimpleCaller(ShowLoginDialogEvent.class);
		}
	
		InputField inputUsername = new InputField(TYPE.TEXT);
		InputField inputPassword = new InputField(TYPE.PASSWORD);
	
		private String getHTML() {
	
			StringBuilder html = new StringBuilder();
	
			html.append("<table border='0'>");
			html.append("<tr>");
			html.append("<td>Username:</td>");
			html.append("<td>" + inputUsername + "</td>");
			html.append("</tr>");
			html.append("<tr>");
			html.append("<td>Password:</td>");
			html.append("<td>" + inputPassword + "</td>");
			html.append("</tr>");
			html.append("</table>");
	
			return html.toString();
		}
	
		@Override
		public JavaScript getJavaScriptResponse() {
	
			JavaScript script = new JavaScript();
	
			script.append("$(\"<div>" + StringUtil.downGradeQuotes(getHTML()) + "</div>\").dialog( {");
			script.append("	autoOpen: true,");
			script.append("	modal: true,");
			script.append("	title: 'Login',");
			script.append("	resizable: false,");
			script.append("	buttons: {");
			script.append("		Ok: function() {");
			script.append(LoginUserEvent.call(inputUsername.getElementID(), inputPassword.getElementID()));
			script.append("		},");
			script.append("		Cancel: function() {");
			script.append("			$( this ).dialog( 'close' );");
			script.append("		}");
			script.append("	}");
	
			script.append("});");
	
			return script;
	
		}
	
	}


Dependencies
===============================================================================

This framework requires the Java-JavaScript-Wrapper found at:

https://github.com/corgrath/Java-JavaScript-Wrapper/



JavaDoc
===============================================================================

http://dl.dropbox.com/u/8183146/persistent/projects/java_ajaxpostbackservlet/javadoc/index.html



Changes
===============================================================================

v1.0 2011-08-21 Christoffer Pettersson

 * Initial release



Report problems
===============================================================================

Please report any problems here:

https://github.com/corgrath/AjaxPostBackServlet/issues/



License
=========================================================================

AjaxPostBackServlet - Copyright 2011 Christoffer Pettersson, christoffer@christoffer.me

Licensed under the Apache License, Version 2.0