/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.apbs;

import java.io.Serializable;

import com.osbcp.apbs.events.AjaxPostBackEvent;
import com.osbcp.jjsw.JavaScript;

/**
 * A specific JavaScript, used to call the <i>$apb</i> JavaScript function that performs the AJAX request.
 * 
 * @author <a href="mailto:christoffer@christoffer.me">Christoffer Pettersson</a>
 */

public abstract class AjaxPostBackCaller extends JavaScript {

	private static final long serialVersionUID = 1L;

	/**
	 * Creates a new instance of an AjaxPostBackCaller instance, used to call the <i>$apb</i> JavaScript function that performs an AJAX request.
	 * 
	 * @param preScript Any JavaScript that should be executed before the AJAX request.
	 * @param postScript Any JavaScript that should be executed before the event's own JavaScript response is executed.
	 * @param eventClass The event that should be triggered.
	 * @param arguments Any arguments that should be sent to the event, should be the same argument types as the event's constructor.
	 */

	public AjaxPostBackCaller(final JavaScript preScript, final JavaScript postScript, final Class<? extends AjaxPostBackEvent> eventClass, final Serializable... arguments) {

		String preScriptString = !JavaScript.isEmpty(preScript) ? preScript.toString(TYPE.DOWNGRADE) : "";
		String postScriptString = !JavaScript.isEmpty(postScript) ? postScript.toString(TYPE.DOWNGRADE) : "";
		String eventName = eventClass.getName();
		String argumentString = argumentsToString(arguments);

		append("$apb('" + preScriptString + "','" + postScriptString + "','" + eventName + "',[" + argumentString + "]);");

	}

	/**
	 * Takes in an array of method arguments, and returns a String comma separated JavaScript friendly list
	 * 
	 * @param arguments Argument values
	 * @return Returns a String comma separated JavaScript friendly list
	 */

	private static String argumentsToString(final Serializable... arguments) {

		StringBuilder output = new StringBuilder();

		// For each argument
		for (int i = 0; i < arguments.length; i++) {

			// Get the argument
			Object argument = arguments[i];

			if (argument == null) {

				// If the argument is null, encode it into a 'null' String to be decoded later
				output.append("'null'");

			} else if (argument instanceof JavaScript) {

				// Since it's a JavaScrip, conjugate it into the JavaScript variable
				output.append(argument);

			} else {

				// URL encode the argument
				String escaped = argument.toString();
				//				try {
				//					escaped = URLEncoder.encode(argument.toString(), "UTF-8");
				//				} catch (UnsupportedEncodingException e) {
				//				}

				// Append it to the URL
				output.append("'" + escaped + "'");

			}

			// Add a colon unless it is the last item
			if (i < (arguments.length - 1)) {
				output.append(", ");
			}

		}

		// Return the trimmed list
		return output.toString().trim();

	}

}