/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.apbs;

import java.io.IOException;
import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.osbcp.apbs.events.AjaxPostBackEvent;
import com.osbcp.jjsw.JavaScript;

/**
 * A specific HttpServlet for AjaxPostBack.
 * 
 * @author <a href="mailto:christoffer@christoffer.me">Christoffer Pettersson</a>
 */

public abstract class AjaxPostBackServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;

	@Override
	public final void doGet(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {

		// Get the data from the query string
		String eventName = request.getParameter("event");
		String[] arguments = request.getParameterValues("argument");

		// Populate the meta data map
		Map<Serializable, Object> metaData = getMetaData(request, response);

		// Get the response script
		JavaScript responseScript = getJavaScriptResponse(eventName, arguments, metaData);

		// Print out the JavaScript 
		printJavaScriptResponse(response, responseScript);

	}

	/**
	 * Logic that invokes the given event with the specified arguments and meta data.
	 * 
	 * @param eventName The class name of the event to be triggered.
	 * @param arguments The constructor arguments to the event.
	 * @param metaData Any meta data that should be included in the event. 
	 * @return A JavaScript response that should be performed by the browser.
	 */

	@SuppressWarnings("unchecked")
	private JavaScript getJavaScriptResponse(final String eventName, final String[] arguments, final Map<Serializable, Object> metaData) {

		// Create a new response container
		JavaScript response = null;

		// Save some variables for logging
		long startTime = System.currentTimeMillis();

		try {

			// Get the event class
			final Class<? extends AjaxPostBackEvent> eventClass = (Class<? extends AjaxPostBackEvent>) Class.forName(eventName);

			// Get a new instance of the event
			Object event = invokeEvent(eventClass, arguments);

			// Cast it as an AjaxPostBackEvent
			AjaxPostBackEvent apb = (AjaxPostBackEvent) event;

			// Set any meta data on the event
			apb.setMetaData(metaData);

			// Get the response from the event
			response = apb.getJavaScriptResponse();

		} catch (final Exception exception) {

			// Calculate the duration
			long duration = System.currentTimeMillis() - startTime;

			// Bubble the error
			onError(eventName, arguments, duration, response);

			// Get the response if any exception occurs
			return getExceptionJavaScriptResponse(eventName, arguments, duration, exception);

		}

		// Calculate the duration
		long duration = System.currentTimeMillis() - startTime;

		// Bubble the success
		onSuccess(eventName, arguments, duration, response);

		// Return the response
		return response;

	}

	/**
	 * Invokes a specific event with specific arguments.
	 * 
	 * @param eventClass The class of the event to be triggered.
	 * @param arguments The constructor arguments to the event.
	 * @return An instance of the event.
	 * @throws Exception If any error occurs.
	 */

	private Object invokeEvent(final Class<? extends AjaxPostBackEvent> eventClass, final String[] arguments) throws Exception {

		if (arguments != null) {

			Constructor<?>[] constructors = eventClass.getConstructors();

			for_each_constructor: for (Constructor<?> constructor : constructors) {

				Class<?>[] constructorTypes = constructor.getParameterTypes();

				// If they don't have the same number of slots, continue to the next constructor
				if (constructorTypes.length != arguments.length) {
					continue;
				}

				// Create an array for the arguments we are going to type cast
				Object[] castedArguments = new Object[constructorTypes.length];

				for (int i = 0; i < arguments.length; i++) {

					try {

						// Try and cast the argument to what the constructor expects
						castedArguments[i] = cast(arguments[i], constructorTypes[i]);

					} catch (ClassCastException exception) {

						// If there is a class cast exception, this is the incorrect constructor
						continue for_each_constructor;

					}

				}

				// Invoke the event using the newly casted arguments
				return constructor.newInstance(castedArguments);

			}

			throw new Exception("Could not find any suitable constructor for the class '" + eventClass.getSimpleName() + "' for the given arguments " + arguments + ".");

		} else {

			// Create an instance of the event with the default empty constructor
			return eventClass.newInstance();

		}

	}

	/**
	 * Casts a specific argument to a specific legal type.
	 * 
	 * @param argument The argument that should be casted.
	 * @param castTo The type the argument should be casted to.
	 * @return The argument casted to a legal type.
	 * @throws ClassCastException If any error occurs.
	 */

	private Object cast(final String argument, final Class<?> castTo) throws ClassCastException {

		try {

			if (castTo == String.class) {
				return argument;
			} else if (castTo == Byte.class || castTo == Byte.TYPE) {
				return Byte.parseByte(argument);
			} else if (castTo == Short.class || castTo == Short.TYPE) {
				return Short.parseShort(argument);
			} else if (castTo == Integer.class || castTo == Integer.TYPE) {
				return Integer.parseInt(argument);
			} else if (castTo == Long.class || castTo == Long.TYPE) {
				return Long.parseLong(argument);
			} else if (castTo == Float.class || castTo == Float.TYPE) {
				return Float.parseFloat(argument);
			} else if (castTo == Double.class || castTo == Double.TYPE) {
				return Double.parseDouble(argument);
			} else if (castTo == Boolean.class || castTo == Boolean.TYPE) {
				return Boolean.valueOf(argument);
			} else if (castTo == Character.class || castTo == Character.TYPE) {
				return argument.charAt(0);
			}

		} catch (Exception e) {
			// Fall through
		}

		throw new ClassCastException("The argument '" + argument + "' could not be casted to anything legal.");

	}

	/**
	 * Writes a specific JavaScript script to the HTTP response stream.
	 * 
	 * @param response The HTTP response.
	 * @param responseScript The JavaScript script that should be written.
	 * @throws IOException If any error occurs.
	 */

	private void printJavaScriptResponse(final HttpServletResponse response, final JavaScript responseScript) throws IOException {

		response.setCharacterEncoding("UTF-8");
		response.setContentType("text/javascript");
		response.getWriter().println(responseScript);

	}

	/**
	 * Used to specific specific meta data that should be included in AjaxPostBackEvents.
	 * 
	 * @param request The HTTP request.
	 * @param response The HTTP response.
	 * @return A map of meta data that should be included in AjaxPostBackEvents.
	 */

	protected abstract Map<Serializable, Object> getMetaData(final HttpServletRequest request, final HttpServletResponse response);

	/**
	 * Returns the JavaScript script that should be sent to the browser if any exception error occurs.
	 * 
	 * @param eventName The class name of the event.
	 * @param arguments The arguments sent to the event.
	 * @param duration The duration in milliseconds of the event logic.
	 * @param exception The exception error that occurred.
	 * @return The JavaScript script that should be sent to the browser if any exception error occurs.
	 */

	protected abstract JavaScript getExceptionJavaScriptResponse(final String eventName, final String[] arguments, final long duration, final Exception exception);

	/**
	 * Triggers when an event is successfully triggered.
	 * 
	 * @param eventName The class name of the event.
	 * @param arguments The arguments sent to the event.
	 * @param duration The duration in milliseconds of the event logic.
	 * @param response The JavaScript response the event returned.
	 */

	protected abstract void onSuccess(final String eventName, final String[] arguments, final long duration, final JavaScript response);

	/**
	 * Triggers when an event failed.
	 * 
	 * @param eventName The class name of the event.
	 * @param arguments The arguments sent to the event.
	 * @param duration The duration in milliseconds of the event logic.
	 * @param response The JavaScript response the event returned.
	 */

	protected abstract void onError(final String eventName, final String[] arguments, final long duration, final JavaScript response);

}
