/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.apbs.annotations;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.ExecutableType;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.type.TypeVisitor;
import javax.lang.model.util.SimpleTypeVisitor6;
import javax.tools.Diagnostic.Kind;

/**
 * The Annotation Processing Tool logic for the @IsAjaxPostBackEvent annotation.
 * 
 * @see @IsAjaxPostBackEvent
 * @see <a href="http://download.oracle.com/javase/6/docs/api/javax/annotation/processing/AbstractProcessor.html">AbstractProcessor</a>
 * @author <a href="mailto:christoffer@christoffer.me">Christoffer Pettersson</a>
 */

@SupportedAnnotationTypes("com.osbcp.apbs.annotations.IsAjaxPostBackEvent")
public class AjaxPostBackAnnotationProcessor extends AbstractProcessor {

	/**
	 * Used to toggle debug mode.
	 */

	public static boolean DEBUG = false;

	/**
	 * AbstractProcessor.process
	 * 
	 * @see <a href="http://download.oracle.com/javase/6/docs/api/javax/annotation/processing/AbstractProcessor.html#process%28java.util.Set,%20javax.annotation.processing.RoundEnvironment%29">AbstractProcessor.process</a>
	 * @param annotations The annotation types requested to be processed.
	 * @param roundEnvironment Environment for information about the current and prior round.
	 * @return Whether or not the set of annotations are claimed by this processor.
	 */

	@Override
	public boolean process(final Set<? extends TypeElement> annotations, final RoundEnvironment roundEnvironment) {

		try {

			StringBuilder debug = new StringBuilder();

			Set<? extends Element> elements = roundEnvironment.getElementsAnnotatedWith(IsAjaxPostBackEvent.class);

			for (final Element classElement : elements) {

				/*
				 * Check the parent class 
				 */

				if (!classElement.getModifiers().contains(Modifier.PUBLIC)) {
					error(classElement, debug, "The event '" + classElement.getSimpleName() + "' has to be public.");
					return true;
				}

				/*
				 * Constructor 
				 */

				List<Element> constructors = getEnclosedElements(debug, classElement, ElementKind.CONSTRUCTOR);
				debug.append("found " + constructors.size() + " constructors\n");

				if (constructors.isEmpty()) {
					error(classElement, debug, "The event '" + classElement.getSimpleName() + "' needs to have an constructor.");
				}

				if (constructors.size() > 1) {
					error(classElement, debug, "The event '" + classElement.getSimpleName() + "' may only have one constructor.");
				}

				Element elementConstructor = constructors.get(0);

				if (!elementConstructor.getModifiers().contains(Modifier.PUBLIC)) {
					error(classElement, debug, "The event '" + classElement.getSimpleName() + "' has to have a public constructor.");
				}

				/*
				 * Get constructor data 
				 */

				MethodData constructorData = getMethodData(elementConstructor);
				debug.append("constructor-data-return-type:'" + constructorData.getReturnType() + "'\n");
				for (TypeMirror paraElemtn : constructorData.getParameterTypes()) {
					debug.append("constructor-data-parameter:" + paraElemtn + "\n");
				}
				for (Modifier modifier : elementConstructor.getModifiers()) {
					debug.append("constructor-modifier:" + modifier.name() + "\n");
				}

				/*
				 * Check for illegal constructor parameter types
				 */

				Object illegalParameterType = getIllegalConstructorParameterType(constructorData.getParameterTypes());
				if (illegalParameterType != null) {
					error(classElement, debug, "The constructor parameter type '" + illegalParameterType.toString() + "' is illegal .");
					return true;
				}

				/*
				 * Check the call method
				 * But only if the class is not abstract
				 */

				if (!classElement.getModifiers().contains(Modifier.ABSTRACT)) {

					List<Element> methods = getEnclosedElements(debug, classElement, ElementKind.METHOD);
					List<Element> methodsCall = new ArrayList<Element>();

					for (Element method : methods) {
						debug.append("method:" + method.getSimpleName() + ":" + method.getKind() + "\n");
						if ("call".equals(method.getSimpleName().toString())) {
							methodsCall.add(method);
						}
					}

					if (methodsCall.isEmpty()) {
						error(classElement, debug, "The event '" + classElement.getSimpleName() + "' needs to have a public and static method namned 'call'.");
					}

					if (methodsCall.size() > 1) {
						error(classElement, debug, "The event '" + classElement.getSimpleName() + "' may only have method called 'call'.");
					}

					Element elementCallMethod = methodsCall.get(0);

					if (!elementCallMethod.getModifiers().contains(Modifier.PUBLIC) && !elementCallMethod.getModifiers().contains(Modifier.STATIC)) {
						error(classElement, debug, "The 'call' method has to be both public and static.");
					}

					/*
					 * Get method data
					 */

					MethodData methodData = getMethodData(elementCallMethod);
					debug.append("method-data-return-type1:'" + methodData.getReturnType() + "'\n");
					debug.append("method-data-return-type2:'" + methodData.getReturnType().toString() + "'\n");
					for (TypeMirror paraElemtn : methodData.getParameterTypes()) {
						debug.append("method-data-parameter1:" + paraElemtn + "\n");
					}

					/*
					 * Check the return type
					 */

					if (!"com.osbcp.apbs.AjaxPostBackCaller".equals(methodData.getReturnType().toString())) {
						error(classElement, debug, "The 'call' method has to return the 'com.osbcp.apbs.AjaxPostBackCaller' type.");
					}

					/* 
					 * Check the parameters
					 */

					if (constructorData.getParameterTypes().size() != methodData.getParameterTypes().size()) {
						error(classElement, debug, getErrorMessageHasToHaveParameters(elementCallMethod, constructorData.getParameterTypes()) + " Incorrect number of parameters.");
						return true;
					}

					// Check the parameters
					for (int i = 0; i < methodData.getParameterTypes().size(); i++) {

						String methodParameterType = methodData.getParameterTypes().get(i).toString();
						String constructorParameterType = constructorData.getParameterTypes().get(i).toString();

						boolean isSameParameterType = methodParameterType.equals(constructorParameterType);
						boolean isJavaScriptType = "com.osbcp.jjsw.JavaScript".equals(methodParameterType);

						debug.append("checking-parameter-" + i + ":" + constructorParameterType + "==" + methodParameterType + "?\n");
						debug.append("isSameParameterType=" + isSameParameterType + "\n");
						debug.append("isJavaScriptType=" + isJavaScriptType + "\n");

						if (!isSameParameterType && !isJavaScriptType) {
							error(classElement, debug, getErrorMessageHasToHaveParameters(elementCallMethod, constructorData.getParameterTypes()));
							return true;
						}

					}

				}

			}

		} catch (final Throwable e) {
			processingEnv.getMessager().printMessage(Kind.ERROR, e.toString());
		}

		return true;

	}

	/**
	 * Checks a list of parameter types for any illegal types used for a constructor.
	 * 
	 * @param parameterTypes The parameter types for a constructor.
	 * @return Any illegal parameter type.
	 */

	private Object getIllegalConstructorParameterType(final List<? extends TypeMirror> parameterTypes) {

		List<String> legalTypes = new ArrayList<String>();
		legalTypes.add("java.lang.String");
		legalTypes.add("java.lang.Integer");
		legalTypes.add("int");
		legalTypes.add("java.lang.Byte");
		legalTypes.add("byte");
		legalTypes.add("java.lang.Short");
		legalTypes.add("short");
		legalTypes.add("java.lang.Long");
		legalTypes.add("long");
		legalTypes.add("java.lang.Float");
		legalTypes.add("float");
		legalTypes.add("java.lang.Double");
		legalTypes.add("double");
		legalTypes.add("java.lang.Boolean");
		legalTypes.add("boolean");
		legalTypes.add("java.lang.Character");
		legalTypes.add("character");

		for (Object parameter : parameterTypes) {

			if (!legalTypes.contains(parameter.toString())) {
				return parameter;
			}

		}

		// No illegal type
		return null;

	}

	/**
	 * Get a fancy error message for incorrect parameters.
	 * 
	 * @param elementCallMethod The method element.
	 * @param list List of method parameter types.
	 * @return A fancy error message for incorrect parameters.
	 */
	private String getErrorMessageHasToHaveParameters(final Element elementCallMethod, final List<? extends TypeMirror> list) {
		if (list.isEmpty()) {
			return "The method '" + elementCallMethod.getSimpleName() + "' should not have any parameters.";
		} else {
			return "The method '" + elementCallMethod.getSimpleName() + "' has to have the parameters '" + StringUtil.toString(list) + "' or switch any parameter to a 'com.osbcp.jjsw.JavaScript' type.";
		}
	}

	/**
	 * Displays the error message, uses the DEBUG variable to determine if the debug message should be displayed as well.
	 * 
	 * @param element The element which caused the error.
	 * @param debug The Debug string.
	 * @param message The error message.
	 */

	private void error(final Element element, final StringBuilder debug, final String message) {

		if (DEBUG) {
			processingEnv.getMessager().printMessage(Kind.ERROR, debug.toString() + "\n\n" + message, element);
		} else {
			processingEnv.getMessager().printMessage(Kind.ERROR, message, element);
		}

	}

	/**
	 * Returns method data of a method element.
	 * 
	 * @param elementMethod The method element to check. 
	 * @return Method data of a method element.
	 */

	private MethodData getMethodData(final Element elementMethod) {

		MethodData data = new MethodData();

		TypeMirror mirror = elementMethod.asType();
		mirror.accept(visitor(data), null);

		return data;

	}

	/**
	 * Method visitor
	 * @param data Method data that should be populated with data.
	 * @return A visitor
	 */

	private TypeVisitor<Boolean, Void> visitor(final MethodData data) {

		return new SimpleTypeVisitor6<Boolean, Void>() {

			public Boolean visitExecutable(ExecutableType t, Void v) {
				data.setReturnType(t.getReturnType());
				data.setParameterTypes(t.getParameterTypes());
				return true;
			}

		};

	}

	/**
	 * Returns a list of enclosed elements of an element.
	 * 
	 * @param debug The debug string.
	 * @param element The parent element. 
	 * @param elementKind The element kind.
	 * @return A list of enclosed elements of an element.
	 */

	private List<Element> getEnclosedElements(final StringBuilder debug, final Element element, final ElementKind elementKind) {

		List<Element> list = new ArrayList<Element>();

		for (Element enclosedElement : element.getEnclosedElements()) {

			if (enclosedElement.getKind() == elementKind) {
				debug.append("adding " + elementKind + ":" + enclosedElement.getSimpleName() + ":" + enclosedElement.getKind() + "\n");
				list.add(enclosedElement);
			}

		}

		return list;

	}

}