/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.jjsw;

import java.io.File;
import java.io.PrintStream;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Generates JavaScript functions to a JavaScript source file based on JavaScriptFunctions.
 * 
 * @author <a href=\"mailto:christoffer@christoffer.me\">Christoffer Pettersson</a>
 *
 */

public final class Generator {

	/**
	 * Private constructor to disallow creating an instance of the class.
	 */

	private Generator() {
	}

	/**
	 * Generates JavaScript functions to a JavaScript source file based on JavaScriptFunctions.
	 * 
	 * @param classes A list of classes which JavaScript methods that should be exported
	 * @param javaScriptFile File where the generated JavaScript source file that should be written
	 * @param addJavaScriptComments If additional comments should be added in the generated JavaScript source file
	 * @throws Exception If any error occurs
	 */

	public static void generateTo(final List<Class<?>> classes, final File javaScriptFile, final boolean addJavaScriptComments) throws Exception {

		// Create an empty list for all the found functions
		Map<String, List<String>> functions = new HashMap<String, List<String>>();

		// For each class
		for (final Class<?> klass : classes) {

			// For each declared method
			for (final Method method : klass.getDeclaredMethods()) {

				/*
				 * Validate
				 */

				if (!Validator.validateMethod(method)) {
					continue;
				}

				// Write debug message
				System.out.println("Found eligible method '" + method.getName() + "'.");

				// Get the parameter types
				Class<?>[] types = method.getParameterTypes();

				// Get instances based on those types
				Object[] parameters = ClassUtil.getPreDefinedParametersByTypes(types);

				// Invoke the method
				JavaScript javaScriptFunction = (JavaScript) method.invoke(null, parameters);

				// Get the code based on the objects
				String rawCode = javaScriptFunction.getCodeRaw();
				System.out.println("Raw code is [ " + rawCode + " ].");

				// Get the injected and escaped code
				String code = javaScriptFunction.toString();
				System.out.println("Injected and escaped code is [ " + code + " ].");

				/*
				 * Fix the function names
				 */

				String functionNameJavaScript = StringUtil.md5(rawCode);
				System.out.println("Function name is '" + functionNameJavaScript + "'.");

				/*
				 * Render JavaScript
				 */

				// Write the JavaScript file
				String javaScriptArgumentList = StringUtil.getJavaScriptArgumentList(types.length);

				StringBuilder functionCode = new StringBuilder();

				if (addJavaScriptComments) {
					functionCode.append("/*\n");
					functionCode.append(" * " + klass.getSimpleName() + "." + method.getName() + "\n");
					functionCode.append(" */\n\n");
				}
				functionCode.append(javaScriptFunction.getJavaScriptObjectName() + "." + functionNameJavaScript + " = function(" + javaScriptArgumentList + ") {\n");
				functionCode.append("	" + code + "\n");
				functionCode.append("}\n");

				if (functions.containsKey(javaScriptFunction.getJavaScriptObjectName())) {
					functions.get(javaScriptFunction.getJavaScriptObjectName()).add(functionCode.toString());
				} else {
					List<String> list = new ArrayList<String>();
					list.add(functionCode.toString());
					functions.put(javaScriptFunction.getJavaScriptObjectName(), list);
				}

			}

		}

		/*
		 * Print everything out
		 */

		// Create the file writer for files
		PrintStream outJavaScript = new PrintStream(javaScriptFile);

		// Loop through all the JavaScript objects
		if (addJavaScriptComments) {
			outJavaScript.println("/*");
			outJavaScript.println(" * Object structures");
			outJavaScript.println(" */\n");
		}

		Iterator<String> javaScriptObjectNames = functions.keySet().iterator();
		while (javaScriptObjectNames.hasNext()) {

			String javaScriptObjectName = javaScriptObjectNames.next();

			outJavaScript.println("function " + javaScriptObjectName + "() {");
			outJavaScript.println("}\n");

		}

		// Loop through all the functions and write them to the file
		for (Map.Entry<String, List<String>> entry : functions.entrySet()) {

			//			String javaScriptClassName = entry.getKey();
			List<String> functionsCodes = entry.getValue();

			//			outJavaScript.println("function " + javaScriptClassName + "() {");
			//			outJavaScript.println("}\n");

			for (String functionCode : functionsCodes) {

				outJavaScript.println(functionCode);

			}

		}

		// Close the writer
		outJavaScript.close();

		/*
		 * Print out the done message
		 */

		System.out.println("Code generation done.");

	}
}