/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.jjsw;

import java.io.Serializable;

/**
 * JavaScript wrapper object.
 *
 * @author @author <a href=\"mailto:christoffer@christoffer.me\">Christoffer Pettersson</a>
 */

public class JavaScript implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * Different output types.
	 * 
	 * @author @author <a href=\"mailto:christoffer@christoffer.me\">Christoffer Pettersson</a>
	 */

	public enum TYPE {

		/**
		 * Downgrades all double quotation marks " into single quotation mark '.
		 */

		DOWNGRADE,

		/**
		 * Escapes all double quotation marks " into HTML character &amp;quot;.
		 */

		ESCAPED;

	}

	private String javaScriptObjectName;
	private StringBuilder code;
	private Object[] parameters;

	/**
	 * Creates an empty JavaScript code instance.
	 */

	public JavaScript() {
		this("");
	}

	/**
	 * Creates a new JavaScript instance based on a specific code. 
	 * 
	 * @param code The JavaScript code as String that should be appended
	 */

	public JavaScript(final String code) {
		this("$jjsw", code);
	}

	/**
	 * Creates a new JavaScript instance based on a specific code. Any additional parameters will be concatenated into the code via the %s annotation.
	 * For example:
	 * String name = "John"; 
	 * JavaScriptFunction function = new JavaScriptFunction("alert(\"%s\");", name ); 
	 * 
	 * @param javaScriptObjectName The JavaScript object name, for example "$test"
	 * @param code The JavaScript code
	 * @param parameters Any parameters that should be used in the JavaScript code. Each parameter should be declared as %s in the code. For example: alert('Hello %s %s!'); where the %s are parameters. 
	 */

	public JavaScript(final String javaScriptObjectName, final String code, final Object... parameters) {

		// Validate the parameters
		Validator.validateJavaScriptArguments(javaScriptObjectName, code);

		// Initialize the the variables
		this.javaScriptObjectName = javaScriptObjectName;
		this.code = new StringBuilder(code);
		this.parameters = parameters;

	}

	/**
	 * Appends a JavaScript code to the existing JavaScript code.
	 * 
	 * @param code The JavaScript code to be appended
	 */

	public void append(final JavaScript code) {
		append(code.toString());
	}

	/**
	 * Appends a String to the JavaScript code.
	 * 
	 * @param code The code to be appended
	 */

	public void append(final String code) {
		this.code.append(code);
	}

	/**
	 * Returns the JavaScript code as a String.
	 * 
	 * @return The JavaScript code as a String.
	 */

	public String toString() {

		// Copy the code to a new variable
		String tempCode = code.toString();

		// Inject the arguments
		tempCode = StringUtil.injectJavaScriptVariables(parameters, tempCode);

		// Escape the JavaScript variables
		tempCode = StringUtil.escapeJavaScriptVaribles(tempCode, parameters.length);

		// Return the injected and escaped code
		return tempCode;

	}

	/**
	 * Outputs the JavaScript based on a specific output type.
	 * 
	 * @param type Type of output
	 * @return JavaScript as a String
	 */

	public String toString(final TYPE type) {

		switch (type) {
			case DOWNGRADE: {
				return this.toString().replaceAll("\"", "'");
			}
			case ESCAPED: {
				return this.toString().replaceAll("\"", "&quot;");
			}
			default: {
				return this.toString();
			}
		}
	}

	/**
	 * Wraps the JavaScript in Script-tag, with character set UTF-8.
	 * 
	 * @return The JavaScript wrapped in a Script-tag, with character set UTF-8
	 * @see http://www.w3schools.com/TAGS/tag_script.asp
	 */

	public String toScriptTag() {
		return toScriptTag("UTF-8");
	}

	/**
	 * Wraps the JavaScript in Script-tag.
	 * 
	 * @param charset The character set to be used.
	 * @return The JavaScript wrapped in a Script-tag
	 * @see http://www.w3schools.com/TAGS/tag_script.asp
	 */

	public String toScriptTag(final String charset) {
		return "<script type=\"text/javascript\" charset=\"" + charset + "\">" + toString() + "</script>";
	}

	/**
	 * Creates a Script-tag to a specific JavaScript source, with charset UTF-8.
	 * 
	 * @param source The source URI to the JavaScript code 
	 * @return a Script-tag to a specific JavaScript source, with charset UTF-8
	 * @see http://www.w3schools.com/TAGS/tag_script.asp
	 */

	public static String toScriptTagSource(final String source) {
		return toScriptTagSource("UTF-8", source);
	}

	/**
	 *  Creates a Script-tag to a specific JavaScript source.
	 *  
	 * @see http://www.w3schools.com/TAGS/tag_script.asp
	 * @param charset The character set to be used.
	 * @param source The source URI to the JavaScript code 
	 * @return The JavaScript HTML tag
	 * 
	 */

	public static String toScriptTagSource(final String charset, final String source) {
		return "<script type=\"text/javascript\" charset=\"" + charset + "\" src=\"" + source + "\" ></script>";
	}

	/**
	 * Checks if the JavaScript is empty.
	 * 
	 * @param html The JavaScript to check
	 * @return if the JavaScript is empty
	 */

	public static boolean isEmpty(final JavaScript html) {
		return html == null || html.toString().trim().length() == 0;
	}

	/**
	 * Returns the raw JavaScript code.
	 * 
	 * @return The raw JavaScript code
	 */

	final String getCodeRaw() {
		return code.toString();
	}

	/**
	 * Returns the JavaScript object name.
	 * 
	 * @return The JavaScript object name
	 */

	final String getJavaScriptObjectName() {
		return javaScriptObjectName;
	}

	/**
	 * Returns the JavaScript code to call the function. JavaScriptFunction needs to be generated via the Generator class. By default a semicolon will be appended at the end.
	 * 
	 * @see Generator 
	 * @return The JavaScript code that calls the function
	 */

	final public JavaScript call() {
		return call(true);
	}

	/**
	 *  Returns the JavaScript code to call the function. JavaScriptFunction needs to be generated via the Generator class.
	 *  
	 * @see Generator 
	 * @param useSemicolon If a semicolon should be appended at the end
	 * @return The JavaScript code that calls the function
	 */

	final public JavaScript call(final boolean useSemicolon) {

		String s = StringUtil.getJavaArgumentList(parameters);
		String functionName = StringUtil.md5(getCodeRaw());

		return new JavaScript(javaScriptObjectName + "." + functionName + "(" + s + ")" + (useSemicolon ? ";" : ""));

	}

}