/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.jjsw;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Utility class for String methods.
 * 
 * @author <a href=\"mailto:christoffer@christoffer.me\">Christoffer Pettersson</a>
 */

final class StringUtil {

	/**
	 * Get the argument list for a JavaScript function.
	 * 
	 * @param nbrOfArguments Number of arguments
	 * @return The argument list
	 */

	final static String getJavaScriptArgumentList(final int nbrOfArguments) {

		// If there number of arguments is 0 or lower, return an empty String
		if (nbrOfArguments < 1) {
			return "";
		}

		/*
		 * Create the string
		 */

		StringBuilder sb = new StringBuilder();

		for (int i = 0; i < nbrOfArguments; i++) {

			sb.append(StringUtil.getCrypticJavaScriptVariableName(i));

			if (i < nbrOfArguments - 1) {

				sb.append(", ");
			}

		}

		return sb.toString();

	}

	/**
	 * Injects the code with JavaScript variables.
	 * 
	 * @param parameters The JavaScript variables
	 * @param code The original JavaScript code
	 * @return Injected JavaScript code
	 */

	final static String injectJavaScriptVariables(final Object[] parameters, String code) {

		// For each parameter
		for (Object parameter : parameters) {

			for (int i = 0; i < parameters.length; i++) {

				if (parameter == parameters[i]) {
					parameter = StringUtil.getCrypticJavaScriptVariableName(i);
					break;
				}

			}

			// Replace the first instance with the appropiate value
			code = code.replaceFirst("\\%s", Matcher.quoteReplacement(parameter.toString()));

		}

		return code;
	}

	/**
	 * Finds the first single or double quotation mark.
	 * 
	 * @param text The text to find the quotation mark in
	 * @return A single or double quation mark
	 * @throws IllegalArgumentException If any error occurs
	 */

	final static char findFirstQuotationMark(String text) throws IllegalArgumentException {

		if (text == null || text.trim().length() == 0) {
			throw new IllegalArgumentException("Text may not be empty.");
		}

		text = text.replace("\\\"", "");
		text = text.replace("\\\'", "");

		int indexSingleQuote = text.indexOf('\'');
		int indexDoubleQuote = text.indexOf('\"');

		if (indexSingleQuote == -1 && indexDoubleQuote == -1) {
			throw new IllegalArgumentException("Could not find any quotation marks in the text [" + text + "].");
		} else if (indexSingleQuote != -1 && indexDoubleQuote == -1) {
			return '\'';
		} else if (indexSingleQuote == -1 && indexDoubleQuote != -1) {
			return '\"';
		} else {
			return indexSingleQuote < indexDoubleQuote ? '\'' : '\"';
		}

	}

	/**
	 * Returns The JavaScript variable.
	 * 
	 * @param order Index order of the variable
	 * @return The JavaScript variable
	 */

	final static String getCrypticJavaScriptVariableName(final int order) {
		return "$" + order + "$";
	}

	/**
	 * Escapes any JavaScript variables in the code if they are in quotation marks.
	 * 
	 * @param code The String to check
	 * @param numberOfVariables Maximum number of known variables that could occur
	 * @return JavaScript code with escaped variables, if they are in quotation marks
	 */

	final static String escapeJavaScriptVaribles(String code, final int numberOfVariables) {

		// Create a list for the quotes
		List<String> quotes = new ArrayList<String>();

		// List of allowed patterns
		String[] patterns = new String[] { "(\'[^\']*\')", "(\"[^\"]*\")" };

		// For each pattern text, try and fetch any quotes
		for (String patternText : patterns) {

			Pattern pattern = Pattern.compile(patternText);
			Matcher matcher = pattern.matcher(code);

			while (matcher.find()) {
				quotes.add(matcher.group(1));
			}

		}

		// For each quote
		for (String originalQuote : quotes) {

			String quote = originalQuote;

			// Loop through all the number of variables
			for (int i = 0; i < numberOfVariables; i++) {

				// Generate the JavaScript variable name 
				String variableName = StringUtil.getCrypticJavaScriptVariableName(i);

				// Check if the variable is inside some kind of quotation marks 
				if (quote.contains(variableName)) {

					// Find the first single or double quotation mark
					char firstQuotationMark = StringUtil.findFirstQuotationMark(code.substring(code.indexOf(variableName)));

					// Wrap the variable in the quotation marks, inside the quote
					quote = quote.replace(variableName, firstQuotationMark + " + " + variableName + " + " + firstQuotationMark);

				}

			}

			// Replace the old quote with the new modified quote
			code = code.replace(originalQuote, quote);

		}

		// Return the code
		return code;

	}

	/**
	 * Get the argument list for a Java method.
	 * 
	 * @param parameters The parameters to be used
	 * @return The argument list
	 */

	final static String getJavaArgumentList(final Object... parameters) {

		// If the list of types is empty, return an empty String
		if (parameters.length == 0) {
			return "";
		}

		/*
		 * Create the string
		 */

		StringBuilder sb = new StringBuilder();

		for (int i = 0; i < parameters.length; i++) {

			if (parameters[i] instanceof String) {
				sb.append("\"" + parameters[i].toString() + "\"");
			} else {
				sb.append(parameters[i].toString());
			}

			if (i < parameters.length - 1) {
				sb.append(", ");
			} else {
				sb.append("");
			}

		}

		return sb.toString();

	}

	/**
	 * Fast and dirty MD5 hasher. Please note that this logic does not always work properly.
	 * 
	 * @param s The String to be hashed
	 * @return Hashed value of the String
	 */

	final static String md5(final String s) {

		if (s == null) {
			throw new NullPointerException();
		}

		if (s.length() == 0) {
			return "";
		}

		try {
			MessageDigest md5 = MessageDigest.getInstance("MD5");
			md5.update(s.getBytes());
			BigInteger hash = new BigInteger(1, md5.digest());
			return hash.toString(16);
		} catch (Exception e) {
			throw new RuntimeException(e);
		}

	}

}