/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 */

package com.osbcp.jjsw;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Arrays;

/**
 * Various validation code.
 * 
 * @author <a href=\"mailto:christoffer@christoffer.me\">Christoffer Pettersson</a>
 */

final class Validator {

	/**
	 * Validates a method to see if it is eligible.
	 * 
	 * @param method The method to validate
	 * @return If the method is eligible
	 */

	final static boolean validateMethod(final Method method) {

		// Check if the method is static
		if (!Modifier.isStatic(method.getModifiers())) {
			System.out.println("Method '" + method.getName() + "' is not eligible. Method is not static.");
			return false;
		}

		// Get the return type of the method
		Class<?> returnType = method.getReturnType();

		// Check if the return type is JavaScript
		if (returnType != JavaScript.class) {
			System.out.println("Method '" + method.getName() + "' is not eligible. Return type is '" + returnType.getCanonicalName() + "', not '" + JavaScript.class.getCanonicalName() + "'.");
			return false;
		}

		if (method.getAnnotation(JavaScriptFunction.class) == null) {
			System.out.println("Method '" + method.getName() + "' is not eligible. Does not have the JavaScriptFunction annotation.");
			return false;
		}

		// Create a list of allowed argument types
		Class<?>[] allowedTypes = new Class<?>[] { String.class, Boolean.class, Boolean.TYPE, Integer.class, Integer.TYPE, Double.class, Double.TYPE };

		// Get the argument types
		Class<?>[] types = method.getParameterTypes();

		// If arguments are specified
		if (types.length > 0) {

			// For each type
			for (final Class<?> type : types) {

				// Check if it is allowed
				if (!Arrays.asList(allowedTypes).contains(type)) {
					System.out.println("Method '" + method.getName() + "' is not eligible. Parameter type '" + type.getCanonicalName() + "' is not allowed.");
					return false;
				}

			}

		}

		return true;

	}

	/**
	 * Validates the JavaScript arguments.
	 * 
	 * @param javaScriptObjectName The JavaScript object name
	 * @param code The JavaScript code
	 */

	final static void validateJavaScriptArguments(final String javaScriptObjectName, final String code) {

		// Validate the JavaScript object name
		if (javaScriptObjectName == null || javaScriptObjectName.trim().length() == 0) {
			throw new IllegalArgumentException("JavaScript object name cannot be empty.");
		}

		// JavaScript object name may not contain white space
		if (javaScriptObjectName.trim().contains(" ")) {
			throw new IllegalArgumentException("Spaces may not be in the JavaScript object name.");
		}

		//		if (code == null || code.trim().length() == 0) {
		//			throw new IllegalArgumentException("JavaScript code cannot be empty.");
		//		}

	}
}