package com.osbcp.requiresmethodannotation;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.TypeElement;
import javax.lang.model.type.ExecutableType;
import javax.lang.model.type.TypeMirror;
import javax.lang.model.type.TypeVisitor;
import javax.lang.model.util.SimpleTypeVisitor6;
import javax.tools.Diagnostic.Kind;

@SupportedAnnotationTypes("com.osbcp.requiresmethodannotation.RequiresStaticMethod")
public class RequiresMethodAnnotationProcessor extends AbstractProcessor {

	@Override
	public boolean process(final Set<? extends TypeElement> annotations, final RoundEnvironment roundEnvironment) {

		try {

			StringBuilder debug = new StringBuilder();

			Set<? extends Element> elements = roundEnvironment.getElementsAnnotatedWith(RequiresStaticMethod.class);

			for (final Element element : elements) {

				/*
				 * Get the annotation information
				 */

				RequiresStaticMethod annotation = element.getAnnotation(RequiresStaticMethod.class);
				String expectedMethodName = annotation.name();
				String expectedReturnType = annotation.returns();
				String[] expectedMethodParameters = annotation.parameters();

				// Add some debug information 
				debug.append("annotation-method-name:" + expectedMethodName + "\n");
				debug.append("annotation-method-return-type:" + expectedReturnType + "\n");
				for (String expectedMethodParameter : expectedMethodParameters) {
					debug.append("annotation-method-expected-parameter:" + expectedMethodParameter + "\n");
				}

				/*
				 * Get all the enclosed methods
				 */

				List<Element> methods = getEnclosedElements(debug, element, ElementKind.METHOD);

				/*
				 * Check all methods 
				 */

				boolean foundMethod = false;

				// For each method
				for (Element methodElement : methods) {

					// Get the method data
					MethodData data = getMethodData(debug, methodElement);

					// Check the name
					debug.append("actual-method-name'" + data.getReturnType() + "'\n");
					if (!methodElement.getSimpleName().equals(expectedMethodName)) {
						error(element, debug, "The class '" + element.getSimpleName() + "' requires a method namned '" + expectedMethodName + "'.");
						break;
					}

					// Check the return type
					if (!data.getReturnType().toString().equals(expectedReturnType)) {
						error(element, debug, "The '" + expectedMethodName + "' method has to return the '" + expectedReturnType + "' type.");
						break;
					}

					/*
					 * Check parameter list 
					 */

					if (data.getParameterTypes().size() != expectedMethodParameters.length) {
						error(element, debug, "wrong parameter length");
						break;
					}

					for (int i = 0; i < expectedMethodParameters.length; i++) {
						if (data.getParameterTypes().get(i).equals(expectedMethodParameters[i])) {
							error(element, debug, "incorrect parameter, expected=" + expectedMethodParameters[i] + " actual=" + data.getParameterTypes().get(i));
							break;
						}
					}

					// Check the 

					// This method is correct
					foundMethod = true;
				}

				debug.append("\n");

				// Show an error if we have not found a method
				if (!foundMethod) {
					error(element, debug, "The 'call' method has to return an 'com.osbcp.apbs.AjaxPostBackCaller' object.");
				}

			}

		} catch (Throwable e) {
			processingEnv.getMessager().printMessage(Kind.ERROR, e.toString());
		}

		return true;

	}

	private void error(final Element element, final StringBuilder debug, final String message) {
		// Debug output (uncomment this to show debug message)
		processingEnv.getMessager().printMessage(Kind.ERROR, debug.toString() + "\n\n" + message, element);
		// Production output (uncomment this for production message)
		//		processingEnv.getMessager().printMessage(Kind.ERROR, message, element);

	}

	private MethodData getMethodData(final StringBuilder debug, final Element elementMethod) {

		MethodData data = new MethodData();

		TypeMirror mirror = elementMethod.asType();
		mirror.accept(tomten(data), null);

		return data;

	}

	private TypeVisitor<Boolean, Void> tomten(final MethodData data) {

		return new SimpleTypeVisitor6<Boolean, Void>() {

			public Boolean visitExecutable(ExecutableType t, Void v) {
				data.setReturnType(t.getReturnType());
				data.setParameterTypes(t.getParameterTypes());
				data.setThrownTypes(t.getThrownTypes());
				return true;
			}

		};

	}

	//	private boolean doesClassContainNoArgsConstructor(final StringBuilder debug, Element el) {
	//		TypeMirror mirror = el.asType();
	//		if (mirror.accept(noArgsVisitor, null)) {
	//			return true;
	//		}
	//		return false;
	//	}
	//
	//	private static final TypeVisitor<Boolean, Void> noArgsVisitor = new SimpleTypeVisitor6<Boolean, Void>() {
	//
	//		public Boolean visitExecutable(ExecutableType t, Void v) {
	//			return t.getParameterTypes().isEmpty();
	//		}
	//	};

	private List<Element> getEnclosedElements(final StringBuilder debug, final Element element, final ElementKind elementKind) {

		List<Element> list = new ArrayList<Element>();

		for (Element enclosedElement : element.getEnclosedElements()) {

			debug.append("enclosed:" + enclosedElement.getSimpleName() + ":" + enclosedElement.getKind() + "\n");

			if (enclosedElement.getKind() == elementKind) {
				debug.append("adding constructor:" + enclosedElement.getSimpleName() + "\n");
				list.add(enclosedElement);
			}

		}

		return list;

	}
}